/*
 * pdftxt - a mupdf-based pdf text extractor
 *
 * Copyright (C) 2012-2013 Ali Gholami Rudi
 *
 * This program is released under the modified BSD license.
 */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "fitz.h"

static void extract_page(fz_text_page *pg, int pnum)
{
	char buf[1 << 12];
	fz_text_block *blk;
	fz_text_line *ln;
	fz_text_span *sp;
	int i;
	for (blk = pg->blocks; blk < pg->blocks + pg->len; blk++) {
		for (ln = blk->lines; ln < blk->lines + blk->len; ln++) {
			char *s = buf;
			for (sp = ln->spans; sp < ln->spans + ln->len; sp++)
				for (i = 0; i < sp->len; i++)
					*s++ = sp->text[i].c;
			*s = '\0';
			printf("%d: %s\n", pnum + 1, buf);
		}
	}
}

static void extract(char *path)
{
	fz_rect rect;
	fz_device *dev, *tdev;
	fz_page *page;
	fz_context *ctx;
	fz_document *pdf;
	fz_text_sheet *sheet;
	fz_text_page *text;
	fz_display_list *page_list;
	int i;
	ctx = fz_new_context(NULL, NULL, FZ_STORE_DEFAULT);
	pdf = fz_open_document(ctx, path);
	if (!pdf)
		return;
	for (i = 0; i < fz_count_pages(pdf); i++) {
		page_list = fz_new_display_list(ctx);
		dev = fz_new_list_device(ctx, page_list);
		if (!(page = fz_load_page(pdf, i)))
			continue;
		fz_run_page(pdf, page, dev, &fz_identity, NULL);

		fz_bound_page(pdf, page, &rect);
		sheet = fz_new_text_sheet(ctx);
		text = fz_new_text_page(ctx, &rect);
		tdev = fz_new_text_device(ctx, sheet, text);
		fz_run_display_list(page_list, tdev, &fz_identity,
					&fz_infinite_rect, NULL);
		extract_page(text, i);
		fz_free_device(tdev);
		fz_free_text_page(ctx, text);
		fz_free_text_sheet(ctx, sheet);
		fz_free_page(pdf, page);
		fz_free_device(dev);
		fz_free_display_list(ctx, page_list);
	}
	fz_close_document(pdf);
	fz_free_context(ctx);
}

int main(int argc, char *argv[])
{
	int i;
	if (argc < 2) {
		printf("usage: %s files\n", argv[0]);
		return 1;
	}
	for (i = 1; i < argc; i++)
		extract(argv[i]);
	return 0;
}
